﻿package packages {
	
    public class ImageContainer {
		
		import flash.events.MouseEvent;
		import flash.events.Event;
		import flash.display.Loader;
		import flash.display.Shape;
		import flash.display.Sprite;
		import flash.net.URLLoader;
		import flash.net.URLRequest;
		
		import packages.Globals;
		import packages.Image;
		
		var XMLLoader:URLLoader;
		var container:Sprite;
		var imagecontainer:Sprite;
		var images:Array;
		var Xpos:Number;
		var Ypos:Number;
		var ScrollHandler:handler;
		var ScrollTrack:track;
		var minScroll:Number;
		var maxScroll:Number;
		var OffsetX:Number;
		var scrollfactor:Number;
		
		//CONSTRUCTOR:
		public function ImageContainer(url:String) {
			var XMLurl:URLRequest = new URLRequest(url);
			XMLLoader = new URLLoader(XMLurl);
			Xpos = Globals.FIRSTPOS;
			Ypos = Globals.SPACING;
			container = new Sprite();
			imagecontainer = new Sprite();
			ScrollHandler = new handler();
			ScrollTrack = new track();
			ScrollHandler.height = Globals.SCROLL_HEIGHT;
			ScrollTrack.height = Globals.SCROLL_HEIGHT;
			XMLLoader.addEventListener("complete", XMLLoaded);
		}
		
		private function XMLLoaded(event:Event):void {
			
			var imagesXML:XML = new XML(XMLLoader.data);
			images = new Array(imagesXML.IMAGE.length());
			var i:int = 0
			for each (var imageXML in imagesXML.IMAGE) {
				var image:Image = new Image("images/" + imageXML.NAME);
				image.SetSize(imageXML.@WIDTH * Globals.THUMB_HEIGHT / imageXML.@HEIGHT, Globals.THUMB_HEIGHT);
				image.SetCoords(Xpos, Ypos);
				Xpos += image.GetWidth() + Globals.SPACING;
				imagecontainer.addChild(image);
				images[i] = image;
				images[i].GetImage().addEventListener(MouseEvent.MOUSE_DOWN, MoveTop);
				i++;
			}
			container.addChild(imagecontainer);
			imagecontainer.buttonMode = true;
			DrawScrollbar();
			DrawMask();
		}
		
		private function DrawScrollbar():void {
			ScrollTrack.x = Globals.FIRSTPOS;
			ScrollTrack.y = Globals.THUMB_HEIGHT + 2 * Globals.SPACING;
			ScrollHandler.x = Globals.FIRSTPOS;
			ScrollHandler.y = Globals.THUMB_HEIGHT + 2 * Globals.SPACING;
			container.addChild(ScrollTrack);
			container.addChild(ScrollHandler);
			minScroll = ScrollHandler.x;
			maxScroll = ScrollTrack.width + Globals.FIRSTPOS;
			scrollfactor = ScrollTrack.width / (Xpos - Globals.FIRSTPOS - Globals.SPACING);
			ScrollHandler.width = scrollfactor * ScrollTrack.width;
			ScrollHandler.buttonMode = true;
			ScrollHandler.addEventListener(MouseEvent.MOUSE_DOWN, StartDrag);
			container.parent.addEventListener(MouseEvent.MOUSE_UP, StopDrag);
			ScrollHandler.addEventListener(MouseEvent.MOUSE_OVER, ScrollOver);
			ScrollHandler.addEventListener(MouseEvent.MOUSE_OUT, ScrollOut);
		}
		
		private function DrawMask():void {
			var maskshape:Shape = new Shape();
			maskshape.graphics.lineStyle(1, 0x000000);
			maskshape.graphics.beginFill(0xff0000);
			maskshape.graphics.drawRect(Globals.FIRSTPOS, 0, ScrollTrack.width, container.stage.stageHeight);
			imagecontainer.mask = maskshape;
		}
		
		//EVENTS:
		private function MoveTop(event:MouseEvent):void {
			var targetImg:Loader = Loader(event.target);
			if(container.getChildIndex(imagecontainer) != (container.numChildren - 1)) {
				container.setChildIndex(imagecontainer, (container.numChildren - 1));
			}
			if(imagecontainer.getChildIndex(targetImg) != (imagecontainer.numChildren - 1)) {
				imagecontainer.setChildIndex(targetImg, (imagecontainer.numChildren - 1));
			}
		}
		
		private function StartDrag(event:MouseEvent):void {
			OffsetX = event.stageX - ScrollHandler.x;
			container.parent.addEventListener(MouseEvent.MOUSE_MOVE, Drag);
		}
		
		private function Drag(event:MouseEvent):void {
			ScrollHandler.alpha = Globals.ALPHA_VALUE;
			ScrollHandler.x = event.stageX - OffsetX;
			imagecontainer.x = (-(event.stageX - OffsetX) + Globals.FIRSTPOS) / scrollfactor;
			if(ScrollHandler.x + ScrollHandler.width >= maxScroll) {
				ScrollHandler.x = maxScroll - ScrollHandler.width;
				imagecontainer.x = -imagecontainer.width + ScrollTrack.width;
			}
			if(ScrollHandler.x <= minScroll) {
				ScrollHandler.x = minScroll;
				imagecontainer.x = -ScrollHandler.x + Globals.FIRSTPOS;
			}
			event.updateAfterEvent();
		}
		
		private function StopDrag(event:MouseEvent):void {
			container.parent.removeEventListener(MouseEvent.MOUSE_MOVE, Drag);
			if((event.stageY < ScrollHandler.y || event.stageY > ScrollHandler.y + ScrollHandler.height) || (event.stageX < ScrollHandler.x || event.stageX > ScrollHandler.x + ScrollHandler.width)) {
				ScrollHandler.alpha = 1;
			}
		}
		
		private function ScrollOver(event:MouseEvent):void { ScrollHandler.alpha = Globals.ALPHA_VALUE; }
		private function ScrollOut(event:MouseEvent):void  { ScrollHandler.alpha = 1; }
		
		//ACCESSORS:
		public function GetImageContainer():Sprite     { return imagecontainer; }
		public function GetImageAt(index:Number):Image { return images[index]; }
		public function GetContainer():Sprite          { return container; }

		//MUTATOR:
		public function SetVisibility(vis:Boolean):void { 
			container.visible = vis;
		}
		
		public function SetWidth(W:Number):void { 
			ScrollTrack.width = W;
			imagecontainer.x = 0;
			DrawScrollbar();
			DrawMask();
		}
		
		public function SetY(Y:Number):void { 
			container.y = Y;
		}
	}
}