﻿package packages {
	
    public class StyleContainer {
		
		import flash.events.MouseEvent;
		import flash.events.Event;
		import flash.display.Sprite;
		import flash.display.Shape;
		import flash.net.URLLoader;
		import flash.net.URLRequest;
		
		import packages.Globals;
		import packages.Style;
		
		var XMLLoader:URLLoader;
		var container:Sprite;
		var stylecontainer:Sprite;
		var styles:Array;
		var Xpos:Number;
		var Ypos:Number;
		var ScrollHandler:handler;
		var ScrollTrack:track;
		var minScroll:Number;
		var maxScroll:Number;
		var OffsetX:Number;
		var scrollfactor:Number;
		
		//CONSTRUCTOR:
		public function StyleContainer(url:String) {
			var XMLurl:URLRequest = new URLRequest(url);
			XMLLoader = new URLLoader(XMLurl);
			Xpos = Globals.FIRSTPOS;
			Ypos = Globals.SPACING;
			container = new Sprite();
			stylecontainer = new Sprite();
			ScrollHandler = new handler();
			ScrollTrack = new track();
			ScrollHandler.height = Globals.SCROLL_HEIGHT;
			ScrollTrack.height = Globals.SCROLL_HEIGHT;
			XMLLoader.addEventListener("complete", XMLLoaded);
		}
		
		private function XMLLoaded(event:Event):void {
			var stylesXML:XML = new XML(XMLLoader.data);
			styles = new Array(stylesXML.STYLE.length());
			var i:int = 0;
			for each (var styleXML in stylesXML.STYLE) {
				if (styleXML.@TYPE == 'A4B') {
					var style:Style = new Style(styleXML);
					style.SetCoords(Xpos, Ypos);
					Xpos += style.GetWidth() + Globals.SPACING;
					stylecontainer.addChild(style);
					styles[i] = style;
					styles[i].addEventListener(MouseEvent.MOUSE_DOWN, MoveTop);
					i++;
				}
			}
			container.addChild(stylecontainer);
			DrawScrollbar();
			DrawMask();
		}
		
		private function DrawScrollbar():void {
			ScrollTrack.x = Globals.FIRSTPOS;
			ScrollTrack.y = Globals.THUMB_HEIGHT + 2 * Globals.SPACING;
			ScrollHandler.x = Globals.FIRSTPOS;
			ScrollHandler.y = Globals.THUMB_HEIGHT + 2 * Globals.SPACING;
			container.addChild(ScrollTrack);
			container.addChild(ScrollHandler);
			minScroll = ScrollHandler.x;
			maxScroll = ScrollTrack.width + Globals.FIRSTPOS;
			scrollfactor = ScrollTrack.width / (Xpos - Globals.FIRSTPOS - Globals.SPACING);
			ScrollHandler.width = scrollfactor * ScrollTrack.width;
			ScrollHandler.buttonMode = true;
			ScrollHandler.addEventListener(MouseEvent.MOUSE_DOWN, StartDrag);
			container.parent.addEventListener(MouseEvent.MOUSE_UP, StopDrag);
			ScrollHandler.addEventListener(MouseEvent.MOUSE_OVER, ScrollOver);
			ScrollHandler.addEventListener(MouseEvent.MOUSE_OUT, ScrollOut);
		}
		
		private function DrawMask():void {
			var maskshape:Shape = new Shape();
			maskshape.graphics.lineStyle(1, 0x000000);
			maskshape.graphics.beginFill(0xff0000);
			maskshape.graphics.drawRect(Globals.FIRSTPOS, 0, ScrollTrack.width, container.stage.stageHeight);
			stylecontainer.mask = maskshape;
		}
		
		//EVENTS:
		private function MoveTop(event:MouseEvent):void {
			if(container.getChildIndex(stylecontainer) != (container.numChildren - 1)) {
				container.setChildIndex(stylecontainer, (container.numChildren - 1));
			}
			if(stylecontainer.getChildIndex(event.target.GetStyle()) != (stylecontainer.numChildren - 1)) {
				stylecontainer.setChildIndex(event.target.GetStyle(), (stylecontainer.numChildren - 1));
			}
		}
		
		private function StartDrag(event:MouseEvent):void {
			OffsetX = event.stageX - ScrollHandler.x;
			container.parent.addEventListener(MouseEvent.MOUSE_MOVE, Drag);
		}
		
		private function Drag(event:MouseEvent):void {
			ScrollHandler.alpha = Globals.ALPHA_VALUE;
			ScrollHandler.x = event.stageX - OffsetX;
			stylecontainer.x = (-(event.stageX - OffsetX) + Globals.FIRSTPOS) / scrollfactor;
			if(ScrollHandler.x + ScrollHandler.width >= maxScroll) {
				ScrollHandler.x = maxScroll - ScrollHandler.width;
				stylecontainer.x = -stylecontainer.width + ScrollTrack.width;
			}
			if(ScrollHandler.x <= minScroll) {
				ScrollHandler.x = minScroll;
				stylecontainer.x = -ScrollHandler.x + Globals.FIRSTPOS;
			}
			event.updateAfterEvent();
		}
		
		private function StopDrag(event:MouseEvent):void {
			container.parent.removeEventListener(MouseEvent.MOUSE_MOVE, Drag);
			if((event.stageY < ScrollHandler.y || event.stageY > ScrollHandler.y + ScrollHandler.height) || (event.stageX < ScrollHandler.x || event.stageX > ScrollHandler.x + ScrollHandler.width)) {
				ScrollHandler.alpha = 1;
			}
		}
		
		private function ScrollOver(event:MouseEvent):void { ScrollHandler.alpha = Globals.ALPHA_VALUE; }
		private function ScrollOut(event:MouseEvent):void  { ScrollHandler.alpha = 1; }
		
		//ACCESSORS:
		public function GetStyleContainer():Sprite     { return stylecontainer; }
		public function GetStyleAt(index:Number):Style { return styles[index]; }
		public function GetContainer():Sprite          { return container; }
		
		//MUTATOR:
		public function SetVisibility(vis:Boolean):void { 
			container.visible = vis;
		}
		
		public function SetWidth(W:Number):void { 
			ScrollTrack.width = W;
			stylecontainer.x = 0;
			DrawScrollbar();
			DrawMask();
		}
		
		public function SetY(Y:Number):void { 
			container.y = Y;
		}
	}
}